Package: mockr
Title: Mocking in R
Version: 0.2.2
Date: 2025-05-01
Authors@R: 
    person(given = "Kirill",
           family = "M\u00fcller",
           role = c("aut", "cre"),
           email = "kirill@cynkra.com")
Description: Provides a means to mock a package function, i.e.,
    temporarily substitute it for testing. Designed as a drop-in
    replacement for the now deprecated 'testthat::with_mock()' and
    'testthat::local_mock()'.
License: MIT + file LICENSE
URL: https://krlmlr.github.io/mockr/, https://github.com/krlmlr/mockr
BugReports: https://github.com/krlmlr/mockr/issues
Imports: rlang, withr
Suggests: covr, fs, knitr, pkgload, rmarkdown, testthat, usethis
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2.9000
NeedsCompilation: no
Packaged: 2025-05-01 15:29:44 UTC; kirill
Author: Kirill Müller [aut, cre]
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2025-05-01 16:00:01 UTC
Built: R 4.6.0; ; 2026-04-25 20:30:33 UTC; unix
