% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mockr-package.R
\docType{package}
\name{mockr-package}
\alias{mockr}
\alias{mockr-package}
\title{mockr: Mocking in R}
\description{
Provides a means to mock a package function, i.e., temporarily substitute it for testing. Designed as a drop-in replacement for the now deprecated 'testthat::with_mock()' and 'testthat::local_mock()'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://krlmlr.github.io/mockr/}
  \item \url{https://github.com/krlmlr/mockr}
  \item Report bugs at \url{https://github.com/krlmlr/mockr/issues}
}

}
\author{
\strong{Maintainer}: Kirill Müller \email{kirill@cynkra.com}

}
\keyword{internal}
